#ifndef OST_H_
#define OST_H_

#include <stddef.h>
#include <stdbool.h>

struct ost;
struct ost_iterator;

struct ost * ost_new(bool (*le)(void * a, void * b), bool (*eq)(void * a, void * b));
void ost_delete(struct ost * ost);

void * ost_insert (struct ost * ost, void * data);
void * ost_search (struct ost * ost, void * data);
void * ost_search_iterative (struct ost * ost, void * data);
void * ost_minimum (struct ost * ost);
void * ost_maximum (struct ost * ost);
unsigned int ost_size (struct ost * ost);

void ost_walk_inorder   (struct ost * ost, void (*op)(void * data));
void ost_walk_preorder  (struct ost * ost, void (*op)(void * data));
void ost_walk_postorder (struct ost * ost, void (*op)(void * data));

struct ost_iterator * ost_begin  (struct ost * ost);
struct ost_iterator * ost_end    (struct ost * ost);
struct ost_iterator * ost_next   (struct ost_iterator * it);
struct ost_iterator * ost_prev   (struct ost_iterator * it);
void *                ost_find   (struct ost_iterator * it, void * data);
void *                ost_remove (struct ost_iterator * it);
void *                ost_value  (struct ost_iterator * it);

unsigned int ost_black_depth(struct ost * ost);
void ost_dump(struct ost * ost, void (*print_val)(void *) );

struct ost_iterator * ost_select(struct ost * ost, unsigned int i);
struct ost_iterator * ost_select_subtree(struct ost_iterator * it, unsigned int i);
unsigned int ost_rank(struct ost_iterator * it);


#endif
